package SimpleLock;

use 5.00500;
use strict;
use IO::Dir;
use Carp;
require Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use SimpleLock ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
%EXPORT_TAGS = ( 'all' => [ qw(
	lock unlock
) ] );

@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

@EXPORT = qw(
	
);
$VERSION = '0.05';

sub new {
    my $class = shift;
    my $self  = {
		 lockdir    => ".",
		 timeout    => 30,
		 expiretime => undef,
		 basename   => "lock",
		 current    => undef,
		 nowarn     => 0,
		 @_ };
    bless $self, $class;
}

sub lock {
    my $self = shift;
    my $lockdir  = IO::Dir->new();
    my $filename = $self->{lockdir} . "/" . $self->{basename};

    unless ( $lockdir->open($self->{lockdir}) ) {
	carp "Can't open lock directory '" . $self->{lockdir} . "'";
	return undef;
    }
    my @filelist = $lockdir->read;
    $lockdir->close;

    my $is_lockfile_exist;
    foreach (@filelist) {
	if ( /^$self->{basename}$/ || /^$self->{basename}\.(\d+)$/ ) {
	    $is_lockfile_exist = 1;
	    last;
	}
    }
    unless ( $is_lockfile_exist ) {
	my $file = open(LOCK, ">$filename");
	close(LOCK);
	carp "WARNING: You should touch '$filename' before first lock."
	    unless $self->{nowarn}
    }

    for (my $i = 0; $i < $self->{timeout}; $i++, sleep 1) {
	return $self->{current}
	    if rename($filename, $self->{current} = "${filename}." . time);
    }
    unless ( defined $self->{expiretime} ) { return undef; }

    # re-read directory.
    $lockdir->open($self->{lockdir});
    @filelist = $lockdir->read;
    $lockdir->close;
    foreach (@filelist) {
	if (/^$self->{basename}\.(\d+)/) {
	    return $self->{current}
		if (time - $1 > $self->{expiretime}
		    and rename($self->{lockdir} . "/${_}",
			       $self->{current} = "${filename}." . time));
	    last;
	}
    }
    undef;
}

sub unlock {
    my $self = shift;

    return $self->{lockdir} . "/" . $self->{basename}
	if rename($self->{current}, $self->{lockdir} . "/" . $self->{basename});
}

sub DESTROY {
    my $self = shift;
    $self->unlock if defined $self->{current};
}

1;
__END__

=head1 NAME

SimpleLock - Perl extension for simple exclusive lock.

=head1 SYNOPSIS

  use SimpleLock;
  my $lock = SimpleLock->new(basename   => "lockfile",
			     lockdir    => "./lock",
			     timeout    => 10,
			     expiretime => 600);
  $lock->lock or die "busy!;
  my $fh = IO::File->new("file");
  # blash, blash, blash...
  undef $fh;       # close
  undef $lock;     # auto unlock when destroy

=head1 DESCRIPTION

Simple exclusive lock by renameing lockfile.

=head2 METHODS

=over 4

=item * new

Create new object.

=over 4

=item - lockdir

Assign directory for create lockfile.
default is ".".

This directory B<MUST> be writable.
(But, should B<NOT> assign world writable directory.
To avoid symlink attack.)

=item - basename

Lockfile's basename. plase touch 1st this file.

=item - timeout

Timeout(sec) for lock. default is 30(sec).

=item - expiretime

Timeout(sec) for force purge lock.
Default is undef(=never).

=item - nowarn

If it is ture, suppress non-serious warning message.

=back

=item * lock

Do lock. return filename for success, undef for false.

=item * unlock

Do unlock.

=back

=head2 EXPORT

=item * default

None.

=item * all:

lock, unlock.

=head1 LICENSE

Same as perl itself.

=head1 AUTHOR

Tatsuki Sugiura E<lt>sugi@nemui.orgE<gt>

=head1 THANKS

HZAKI Hiroki! (http://www.din.or.jp/~ohzaki/perl.htm#File_Lock)

=head1 SEE ALSO

L<perl>.

=cut
